Package: bsts
Version: 0.9.10
Date: 2024-01-16
Title: Bayesian Structural Time Series
Author: Steven L. Scott <steve.the.bayesian@gmail.com>
Maintainer: Steven L. Scott <steve.the.bayesian@gmail.com>
Description: Time series regression using dynamic linear models fit using
  MCMC. See Scott and Varian (2014) <DOI:10.1504/IJMMNO.2014.059942>, among many
  other sources.
Depends: BoomSpikeSlab (>= 1.2.6), zoo (>= 1.8), xts, Boom (>= 0.9.13),
        R(>= 3.4.0)
Suggests: testthat
LinkingTo: Boom (>= 0.9.13)
License: LGPL-2.1 | MIT + file LICENSE
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-01-16 23:31:15 UTC; steve
Repository: CRAN
Date/Publication: 2024-01-17 13:02:07 UTC
Built: R 4.4.0; x86_64-apple-darwin10.8.0; 2024-05-01 09:50:58 UTC; unix
