Package: bsts
Version: 0.9.6
Date: 2021-04-06
Title: Bayesian Structural Time Series
Author: Steven L. Scott <steve.the.bayesian@gmail.com>
Maintainer: Steven L. Scott <steve.the.bayesian@gmail.com>
Description: Time series regression using dynamic linear models fit using
  MCMC. See Scott and Varian (2014) <DOI:10.1504/IJMMNO.2014.059942>, among many
  other sources.
Depends: BoomSpikeSlab (>= 1.2.4), zoo (>= 1.8), xts, Boom (>= 0.9.7),
        R(>= 3.4.0)
Suggests: testthat
LinkingTo: Boom (>= 0.9.7)
License: LGPL-2.1 | file LICENSE
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2021-04-07 04:18:51 UTC; steve
Repository: CRAN
Date/Publication: 2021-04-07 09:40:08 UTC
Built: R 3.6.3; x86_64-w64-mingw32; 2021-04-07 12:32:25 UTC; windows
Archs: x64
